"""
Plugin system for NextRPG.

This module provides plugin functions that extend the functionality
of NextRPG games. It includes event handlers and utility functions
that can be registered and used throughout the game framework.

The plugin system features:
- Event registration and handling
- Character and scene interaction
- Message display functionality
- Integration with RPG event system

Example:
    ```python
    from nextrpg.plugins import say
    from nextrpg.character_on_screen import CharacterOnScreen

    # Register a say event
    @register_rpg_event
    def talk_to_npc():
        say(character, "Hello, how are you?")
    ```
"""

from typing import Any

from nextrpg.character_on_screen import CharacterOnScreen
from nextrpg.coordinate import Coordinate
from nextrpg.model import export
from nextrpg.rpg_event import register_rpg_event
from nextrpg.scene import Scene


@export
@register_rpg_event
def say(
    character_or_scene: CharacterOnScreen | Scene,
    message: str,
    arg: Coordinate | None = None,
    **kwargs: Any
) -> None:
    """
    Character says a message.

    This function creates a say event that displays a message
    from a character or scene. It's registered as an RPG event
    and can be used in event scripts.

    Arguments:
        `character_or_scene`: The character or scene that will
            say the message.

        `message`: The message to say.

        `arg`: Optional coordinate argument for positioning.
            Defaults to `None`.

        `**kwargs`: Additional keyword arguments passed to the
            say event.

    Returns:
        `None`: For user code (RpgEventSpec), `say` returns no result
            (from the `SayEvent` scene). For `Npcs`/`MapScene` to yield
            the scene coroutine, it returns an `RpgEventCallable`.

    Example:
        ```python
        from nextrpg.plugins import say
        from nextrpg.character_on_screen import CharacterOnScreen

        # Character says a message
        say(character, "Hello, adventurer!")

        # Scene says a message
        say(scene, "You found a treasure chest!")

        # Say with positioning
        say(character, "Look over here!", Coordinate(100, 200))
        ```
    """
    from nextrpg.say_event import SayEvent

    return lambda generator, scene: SayEvent(
        generator=generator,
        scene=scene,
        character_or_scene=character_or_scene,
        message=message,
        arg=arg,
        **kwargs
    )
